<?php
// $Id: wsapi.inc,v 1.1.2.2 2009/05/16 05:27:11 brmassa Exp $
/**
 * @author Web Services Dev Team
 * @file
 *   Show the help pages.
 */

/**
 * The main API Help page
 */
function _wsapi_api() {
  $settings = variable_get('wsapi_settings', array());

  module_load_include('inc', 'webservices');
  $services = webservices_service_get_all();
  foreach ($services as $service_data) {
    $service = &$service_data['#method'];
    if (!empty($settings['service'][$service])) {
      $group = explode('.', $service_data['#method']);
      $show['service'][$group[0]][$service] = $service;
    }
  }
  if (isset($show['service'])) {
    $show['service']['#title'] = t('Services');
  }

  // Servers
  foreach (module_implements('webservices_server_info') as $module) {
    $method = module_invoke($module, 'webservices_server_info');
    if (!empty($settings['server'][$module])) {
      $show['server'][$module] = $method['#name'];
    }
  }
  if (isset($show['server'])) {
    $show['server']['#title'] = t('Servers');
  }

  // Authentication Methods
  foreach (module_implements('webservices_auth_info') as $module) {
    $method = module_invoke($module, 'webservices_auth_info');
    if (!empty($settings['authentication'][$module])) {
      $show['authentication'][$module] = $method['#name'];
    }
  }
  if (isset($show['authentication'])) {
    $show['authentication']['#title'] = t('Authentications');
  }

  return theme('wsapi', $show);
}

/**
 * The Authentication API Help page
 */
function _wsapi_authentication($item_name) {
  $settings = variable_get('wsapi_settings', array());
  if (!empty($settings['authentication'][$item_name])) {
    $auth = module_invoke($item_name, 'webservices_auth_info');

    // Filter the method data, avoinding exposing sensitive data to
    // theme functions
    $item = array(
      '#name'           => $auth['#name'],
    );
  }

  return theme('wsapi_authentication', $item);
}

/**
 * The Server API Help page
 */
function _wsapi_server($item_name) {
  $settings = variable_get('wsapi_settings', array());
  if (!empty($settings['server'][$item_name])) {
    $server = module_invoke($item_name, 'webservices_server_info');

    // Filter the method data, avoinding exposing sensitive data to
    // theme functions
    $item = array(
      '#name'           => $server['#name'],
      '#description'    => $server['#description'],
      '#path'           => $server['#path'],
      '#example'        => $server['#example'],
    );
  }

  return theme('wsapi_server', $item);
}

/**
 * The Service API Help page
 */
function _wsapi_service($item_name) {
  $settings = variable_get('wsapi_settings', array());
  if (!empty($settings['service'][$item_name])) {
    module_load_include('inc', 'webservices');
    $service = _webservices_service_get($item_name);

    // Get all parameters data
    if (isset($service['#args'])) {
      foreach ($service['#args'] as $param) {
        $params[] = array(
          '#description'    => empty($param['#description']) ? NULL : $param['#description'],
          '#optional'       => empty($param['#optional']) ? NULL : $param['#optional'],
          '#name'           => $param['#name'],
          '#type'           => $param['#type'],
        );
      }
    }

    // Filter the method data, avoinding exposing sensitive data to
    // theme functions
    $item = array(
      '#authentication' => (($service['#access callback'] === TRUE) ? NULL : TRUE),
      '#description'    => isset($service['#description']) ? $service['#description'] : NULL,
      '#name'           => $service['#method'],
      '#params'         => $params,
      '#response'       => empty($service['#return']) ? NULL : $service['#return']
    );
  }

  return theme('wsapi_service', $item);
}

/**
 * The main API Help page
 *
 * @ingroup themable
 */
function theme_wsapi($show) {
  if (empty($show)) {
    return t('No API items are open for you to see.');
  }

  $output = '';
  foreach ($show as $section_id => $section) {
    $section_data = '';
    foreach (element_children($section) as $group) {
      if (is_array($section[$group])) {
        $section_data .= '<h3>'. $group .'</h3>';
        foreach (element_children($section[$group]) as $unit) {
          $links[] = l($section[$group][$unit], 'webservices/api/'. $section_id .'/'. $unit) .'<br/>';
        }
        $section_data .= theme('item_list', $links);
        $links = array();
      }
      else {
        $links[] = l($section[$group], 'webservices/api/'. $section_id .'/'. $group) .'<br/>';
      }
    }
    if (empty($section_data)) {
      $section_data .= theme('item_list', $links);
      $links = array();
    }
    $output .= theme('fieldset', array('#title' => $section['#title'], '#children' => $section_data));
  }

  return $output;
}

/**
 * The Authentication API Help page
 *
 * @ingroup themable
 */
function theme_wsapi_authentication($item) {
  // Page title
  if (!is_array($item)) {
    return t('This authentication does not exist.');
  }

  // Page title
  drupal_set_title(t('Authentication: %auth', array('%auth' => $item['#name'])));

  // Description
  if (isset($item['#description'])) {
    $output .= '<p>'. $item['#description'] .'</p>';
  }
  else {
    $output = '';
  }

  return $output;
}

/**
 * The Server API Help page
 *
 * @ingroup themable
 */
function theme_wsapi_server($item) {
  if (!is_array($item)) {
    return t('This server does not exist.');
  }

  // Page title
  drupal_set_title(t('Server: %server', array('%server' => $item['#name'])));

  // Description
  if (!empty($item['#description'])) {
    $output .= '<p>'. $item['#description'] .'</p>';
  }
  else {
    $output = '';
  }

  // URL
  $output .= '<h3>'. t('Path') .'</h3>';
  $output .= '<p>'. t('The %server server URL is:', array('%server'=> $item['#name'])) .
    ' <code>'. url('webservices/'. $item['#path'], array('absolute' => TRUE)) .'</code></p>';

  // Example
  if (!empty($item['#example'])) {
    $output .= '<h3>'. t('Example') .'</h3>';
    $output .= '<p>'. t('To request the test.echo service, send an !type like this:', array('!type' => $item['#example']['#type'])) .'<br/>';
    $output .= '<code>'. $item['#example']['#test'].'</code></p>';

    if (!empty($item['#example']['#note'])) {
      $output .= '<p>'. $item['#example']['#note'].'</p>';
    }
  }

  // Response Formats
  if (!empty($item['#formats'])) {
    $output .= '<h3>'. t('Response Formats') .'</h3>';
    foreach ($item['#formats'] as $format => $format_name) {
      $formats[] = l($format_name, 'webservices/formats/'. $format, array('absolute' => TRUE));
    }
    $output .= theme('item_list', $formats);
  }

  return $output;
}

/**
 * The Service API Help page
 *
 * @ingroup themable
 */
function theme_wsapi_service($item) {
  if (!is_array($item)) {
    return t('This service does not exist.');
  }

  // Page title
  drupal_set_title(t('Service: %service', array('%service' => $item['#name'])));
  $output = '';

  // Description
  if (isset($item['#description'])) {
    $output .= '<p>'. $item['#description'] .'</p>';
  }

  // Authentication
  $output .= '<h3>'. t('Authentication') .'</h3>';
  $output .= '<p>'. (empty($item['#authentication']) ? t('Not required') : t('Required')) .'</p>';

  // Parameters
  $output .= '<h3>'. t('Parameters') .'</h3>';
  if (isset($item['#params'])) {
    foreach ($item['#params'] as $param) {
      $output .= '<dl><strong>'. $param['#name'] .'</strong> <em>'. $param['#type'] .'</em>';
      if (!empty($param['#optional'])) {
        $output .= ' ('. t('Optional') .')';
      }
      $output .= '<dt>'. $param['#description'] .'</dt></dl>';
    }
  }
  else {
    $output .= '<p>'. t('None') .'</p>';
  }

  // Response
  $output .= '<h3>'. t('Response') .'</h3>';
  $output .= '<p>'. (empty($item['#response']) ? t('None') : $item['#response']) .'</p>';

  return $output;
}
